/* decode.c             read codes from infile and construct array
 *
 * Programmer: R. White         Date: 9 May 1991
 *
 * Modified by J. Doggett, 7 December 1993,
 *         to add <stdlib.h> include for standard function prototypes.
 *         to add <string.h> include for string function prototypes.
 *
 * Modified by J. Doggett, 15 December 1993,
 *         put in char casts for code_magic because in general, a hex
 *         value won't fit in a char, so the acc compiler would complain
 *         about it. It's happy now.
 *
 * Modified by J. Doggett, 18 May 1995,
 *         Added checkCompressedTileFormat function for checking "magic code"
 *         of tile file, and included new decode.h header file.
 */

/******************************************************************************
* Copyright (c) 1993, 1994, Association of Universities for Research in       
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "qfile.h"

#include "decode.h"

extern void dodecode();

extern void
decode(infile,a,nx,ny,scale)
FILE *infile;           /* input file (already opened)          */
int **a;                /* address of output array [nx][ny]     */
int *nx,*ny;            /* size of output array                 */
int *scale;             /* scale factor for digitization        */
{
int i, nel, sumall;
unsigned char nbitplanes[3];
char tmagic[2];

        /*
         * read magic value in first two bytes
         */
        qread(infile, tmagic, sizeof(tmagic));
/*      if (memcmp(tmagic,code_magic,sizeof(code_magic)) != 0) { */
        if (!checkCompressedTileFormat(tmagic)) {
                fprintf(stderr, "bad file format\n");
                exit(-1);
        }
        *nx =readint(infile);   /* x size of image               */
        *ny =readint(infile);   /* y size of image               */
        *scale=readint(infile); /* scale factor for digitization */
        /*
         * allocate memory for array
         */
        nel = (*nx) * (*ny);
        *a = (int *) malloc(nel*sizeof(int));
        if (*a == (int *) NULL) {
                fprintf(stderr, "decode: insufficient memory\n");
                exit(-1);
        }
        sumall=readint(infile);                        /* sum of all pixels  */
        qread(infile, nbitplanes, sizeof(nbitplanes)); /* # bits in quadrant */
        dodecode(infile, *a, *nx, *ny, nbitplanes);
        /*
         * put sum of all pixels back into pixel 0
         */
        (*a)[0] = sumall;
}

/* See if magic value in tile is correct.  Return 1 if it is. Otherwise, 0. */

int checkCompressedTileFormat(tmagic)
char tmagic[2];
{
   char magic0 = (char) 0xDD;
   char magic1 = (char) 0x99;

   if ((tmagic[0] != magic0) ||
       (tmagic[1] != magic1)   )
      return 0;
   else
      return 1;
}
