# CHECK_CFITSIO
#------------------
# Checks for the cfitsio library and header files.
AC_DEFUN([CHECK_CFITSIO],
[

    AC_MSG_CHECKING([for cfitsio])

    cfitsio_check_header="fitsio.h"
    cfitsio_check_lib="libcfitsio.a"

    cfitsio_includes=""
    cfitsio_libraries=""

    AC_ARG_WITH(cfitsio,
                AC_HELP_STRING([--with-cfitsio],
                               [location where cfitsio is installed]),
                [
                    with_cfitsio_includes=$withval/include
                    with_cfitsio_libs=$withval/lib
                ])

    AC_ARG_WITH(cfitsio-includes,
                AC_HELP_STRING([--with-cfitsio-includes],
                               [location of the cfitsio header files]),
                with_cfitsio_includes=$withval)

    AC_ARG_WITH(cfitsio-libs,
                AC_HELP_STRING([--with-cfitsio-libs],
                               [location of the cfitsio library]),
                with_cfitsio_libs=$withval)

    AC_ARG_ENABLE(cfitsio-test,
                  AC_HELP_STRING([--disable-cfitsio-test],
                                 [disables checks for the cfitsio library and headers]),
                  enable_cfitsio_test=$enableval,
                  enable_cfitsio_test=yes)


    if test "x$enable_cfitsio_test" = xyes; then

        # Check for the cfitsio includes

        if test -z "$with_cfitsio_includes"; then
            cfitsio_incdirs="$HOME/local/include /opt/cfitsio/include /usr/local/include /usr/include"
            test -n "$CFITSIODIR" && cfitsio_incdirs="$CFITSIODIR/include \
                $cfitsio_incdirs"
        else
            cfitsio_incdirs="$with_cfitsio_includes"
        fi

        ESO_FIND_FILE($cfitsio_check_header, $cfitsio_incdirs,
                      cfitsio_includes)


        # Check for the cfitsio library

        if test -z "$with_cfitsio_libs"; then
            cfitsio_libdirs="$HOME/local/lib /opt/cfitsio/lib /usr/local/lib /usr/lib"
            test -n "$CFITSIODIR" && cfitsio_libdirs="$CFITSIODIR/lib \
                $cfitsio_libdirs"
        else
            cfitsio_libdirs="$with_cfitsio_libs"
        fi

        ESO_FIND_FILE($cfitsio_check_lib, $cfitsio_libdirs,
                      cfitsio_libraries)


        if test x"$cfitsio_includes" = xno || \
            test x"$cfitsio_libraries" = xno; then
            cfitsio_notfound=""

            if test x"$cfitsio_includes" = xno; then
                if test x"$cfitsio_libraries" = xno; then
                    cfitsio_notfound="(headers and libraries)"
                else
                    cfitsio_notfound="(headers)"
                fi
            else
                cfitsio_notfound="(libraries)"
            fi

            AC_MSG_ERROR([cfitsio $cfitsio_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([OK])
            AC_MSG_RESULT([cfitsio libraries in $cfitsio_libraries])
            AC_MSG_RESULT([cfitsio headers in $cfitsio_includes])
        fi

        # Set up the symbols

        CFITSIO_INCLUDES="-I$cfitsio_includes"
        CFITSIO_LDFLAGS="-L$cfitsio_libraries"
        #LIBCFITSIO="-lcfitsio"
        LIBCFITSIO="$cfitsio_libraries/libcfitsio.a"
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([cfitsio checks have been disabled! This package may not build!])
        CFITSIO_INCLUDES=""
        CFITSIO_LDFLAGS=""
        LIBCFITSIO=""
    fi

    AC_SUBST(CFITSIO_INCLUDES)
    AC_SUBST(CFITSIO_LDFLAGS)
    AC_SUBST(LIBCFITSIO)

])
