/* static char sccsid[] = "@(#) ESO/ST-ECF dss/src/http.c	1.0	05/17/95"; */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.TYPE		Module
.NAME		http.c
.LANGUAGE	C
.AUTHOR		M Albrecht [ESO]
.COMMENTS	Basic http client functions for the DSS package.

.VERSION 1.0    17-May-1995 Creation.
------------------------------------------------------------------------*/

#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/socket.h>
#include	<netinet/in.h>
#include	<netdb.h>
#include	<arpa/inet.h>

#define	DSS_HOST        "acj.hq.eso.org"
#define	DSS_PORT	8080

#define	MAXLINE		512
#define	BUFSIZE		4096


http_get(filein, fileout)
/*++++++++++++
.PURPOSE Connect to HTTP server and send a GET method request,
         read and skip http header (ends with a blank line)
         then read the file from the socket and write it to the local disk.
.RETURNS Nothing, exits upon socket errors (not very good).
--------------*/
char	*filein, *fileout;
{
     int sockfd;
     struct sockaddr_in	serv_addr;
     int n, flength, port;
     char sendline[BUFSIZE], recvline[MAXLINE + 1], buffer[BUFSIZE];
     char *p, *h, *getenv(), *strstr();
     FILE *fo;
     struct hostent *host;

     /* Get the DSS host address translated and the port number */
     if( !(h=getenv("DSS_HOST")) ) return(0);

     if( !(host = gethostbyname(h)) ){
	  perror("Can't find DSS host address");
	  exit(1);
     }

     if( (p=getenv("DSS_PORT")) ) port = atoi(p);
     else  port = DSS_PORT;

     /*
      * Fill in the structure "serv_addr" with the address of the
      * server that we want to connect with.
      */

     memset((char *) &serv_addr, '\0', sizeof(serv_addr));
     memcpy(&serv_addr.sin_addr,
	    host->h_addr_list[0], host->h_length);
     serv_addr.sin_family = AF_INET;
     serv_addr.sin_port   = htons(port);
     
     /*
      * Open a TCP socket (an Internet stream socket).
      */
     
     if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	  perror("Can't open stream socket");
	  exit(1);
     }
     
     /*
      * Connect to the server.
      */
     
     if (connect(sockfd, (struct sockaddr *) &serv_addr,
		                    sizeof(serv_addr)) < 0) {
	  perror("Can't connect to DSS server");
	  exit(1);
     }

     /* 
      * Send the GET method 
      */

     sprintf(sendline, "GET %s HTTP/1.0\nUser-Agent: dss\n\n", filein);
     n = strlen(sendline);
     if (writen(sockfd, sendline, n) != n) {
	  perror("Write error on DSS socket");
	  exit(1);
     }

     /* 
      * Read the http header: ends with a blank line 
      */

     flength = 0;
     while ( (n = readline(sockfd, recvline, MAXLINE)) > 0 ) {
	  if(n == 1 || strstr(recvline, "\015\012") == recvline) break;

	  { char *q;
	    for(q=recvline; *q ; q++) *q = (char)tolower((int)*q);
          }

	  if(strstr(recvline, "content-length:")) 
	    flength = atoi(&recvline[15]);
     }
     
     /* 
      * Now read the file from socket and write it to local disk
      */

     if(flength) {
	  if( (fo = fopen(fileout, "w")) == NULL) {
	       perror("Can't open scratch file");
	       exit(1);
	  }
	  while ( (n = read(sockfd, buffer, BUFSIZE)) > 0 ) {
	       fwrite(buffer, n, 1, fo);
	       flength -= n;
	  }
	  fclose(fo);
	  if(flength > 0)
	    fprintf(stderr, 
		    "Uncomplete DSS file: %d bytes missing.\n", flength);
     } else {
	  fprintf(stderr, "DSS Server is down.\n");
	  exit(1);
     }
     
     close(sockfd);
}

http_cgi(url)
/*++++++++++++
.PURPOSE Connect to HTTP server and send a GET method request
         meant as a message to a cgi-bin script.
         Don't expect answer.
.RETURNS Nothing, exits upon socket errors (not very good).
--------------*/
char	*url;
{
     int sockfd;
     struct sockaddr_in	serv_addr;
     int n, flength, port;
     char sendline[BUFSIZE], recvline[MAXLINE + 1], buffer[BUFSIZE];
     FILE *fo;
     char h[1024], get[1000];
     char *p, *getenv();
     struct hostent *host;

     /* Get the DSS host address translated and the port number */
     port = 80;
     if (sscanf(url, "http://%31[^:/]:%d%1000s", h, &port, get) != 3 &&
	 sscanf(url, "http://%31[^/]%1000s", h, get) < 1) {
          return(0);
     }
     
     if( !(host = gethostbyname(h)) ){
          perror("Can't find DSS_LOG host address");
	  exit(1);
     }

     /*
      * Fill in the structure "serv_addr" with the address of the
      * server that we want to connect with.
      */
     
     memset((char *) &serv_addr, '\0', sizeof(serv_addr));
     memcpy(&serv_addr.sin_addr,
	    host->h_addr_list[0], host->h_length);
     serv_addr.sin_family      = AF_INET;
     serv_addr.sin_port        = htons(port);
     
     /*
      * Open a TCP socket (an Internet stream socket).
      */
     
     if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	  perror("Can't open stream socket");
	  exit(1);
     }
     
     /*
      * Connect to the server.
      */
     
     if (connect(sockfd, (struct sockaddr *) &serv_addr,
		                    sizeof(serv_addr)) < 0) {
	  perror("Can't connect to DSS server");
	  exit(1);
     }
     
     /* 
      * Send the GET method 
      */
     sprintf(sendline, "GET %s HTTP/1.0\nUser-Agent: dss\n\n", get);
     n = strlen(sendline);
     if (writen(sockfd, sendline, n) != n) {
	  perror("Write error on socket (http_cgi)");
	  exit(1);
     }

     close(sockfd);
}

int readline(fd, ptr, maxlen)
/*++++++++++++
.PURPOSE Read the line one byte at a time,
         looking for the newline.  We store the newline in the buffer,
	 then follow it with a null (the same as fgets(3)).
	 Not very efficient but usefull for sockets.
.RETURNS The number of characters up to, but not including,
	 the null (the same as strlen(3)) or < 0 upon errors.
.REMARKS Taken from Stevens, "Unix Network Programming".
--------------*/
register int	fd;
register char	*ptr;
register int	maxlen;
{
	int	n, rc;
	char	c;

	for (n = 1; n < maxlen; n++) {
		if ( (rc = read(fd, &c, 1)) == 1) {
			*ptr++ = c;
			if (c == '\n')
				break;
		} else if (rc == 0) {
			if (n == 1)
				return(0);	/* EOF, no data read */
			else
				break;		/* EOF, some data was read */
		} else
			return(-1);	/* error */
	}

	*ptr = 0;
	return(n);
}

int writen(fd, ptr, nbytes)
/*++++++++++++
.PURPOSE Write "n" bytes to a descriptor.
         Use in place of write() when fd is a stream socket.
.RETURNS The number of characters written or <0 upon error.
.REMARKS Taken from Stevens, "Unix Network Programming".
--------------*/
register int	fd;
register char	*ptr;
register int	nbytes;
{
	int	nleft, nwritten;

	nleft = nbytes;
	while (nleft > 0) {
		nwritten = write(fd, ptr, nleft);
		if (nwritten <= 0)
			return(nwritten);		/* error */

		nleft -= nwritten;
		ptr   += nwritten;
	}
	return(nbytes - nleft);
}





